#ifndef Hash_h
#define Hash_h

#include "Data.h"


template<typename _T>
inline void hashCombine(std::size_t& o_seed, const _T& p_val) {
    o_seed ^= std::hash<_T>()(p_val) + 0x9e3379b9 + (o_seed << 6) + (o_seed >> 2);
}

template<typename _T>
inline void hashVal(std::size_t& o_seed, const _T& p_val) {
    hashCombine(o_seed, p_val);
}

template<typename _T, typename..._Types>
inline void hashVal(std::size_t& o_seed, const _T& p_val, const _Types& ...p_args) {
    hashCombine(o_seed, p_val);
    hashVal(o_seed, p_args...);
}

template<typename ... _Types>
inline std::size_t hashVal(const _Types& ...p_args) {
    std::size_t seed = 0;
    hashVal(seed, p_args...);
    return seed;
}

struct hash {
    size_t operator () (const Edge& p_edge) const {
        
        //!!! Данные в hashVal должны передаваться в отсортированном виде.
        //!!! Иначе ребра (1,2) и (2,1) имеют разные хеш-значения, хотя это одно ребро.
        
        return hashVal(p_edge.nodes_of_edge.first, p_edge.nodes_of_edge.second);
    }
};
#endif